/*
  Copyright (C) 2011 Craciun Dan <craciun.dan@tuxarena.com>

  This file is part of NeonView.

  NeonView is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.

  NeonView is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with NeonView.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "header.h"

void read_config (TM *tm) {
	FILE *fd;
	int ignore = 0;
	char opt[257], val[257];
	int isopt = 1, isval = 0;
	int oi, vi; /* opt index, val index */
	char c;

	fd = fopen(tm->tcfg->config_file, "rt");
	if (!fd) {
		fprintf(stderr, "ERROR: Could not read config file.\n");
		return;
	}

	c = fgetc(fd);
	while (c != EOF) {
		isopt = 1; isval = 0;
		oi = 0; vi = 0;
		ignore = 0;
		while (c != '\n') {
			if (ignore == 0) {
				if (c == '#') {
					ignore = 1;
				} else {
					if (c != '=' && isopt == 1) {
						opt[oi] = c;
						oi++;
					}
					if (c != '=' && isval == 1) {
						val[vi] = c;
						vi++;
					}
					if (c == '=') {
						isopt = 0; isval = 1;
					}
				}
			}
			c = fgetc(fd);
		}
		opt[oi] = '\0';
		val[vi] = '\0';
		lc(opt);

		int j;
		for (j = 0; j < vi; j++) {
			if (val[j] == ' ') {
				val[j] = '\0';
			}
		}

		if (strcmp(opt, "window_width") == 0) {
			tm->tcfg->window_width = atoi(val);
		}
		if (strcmp(opt, "window_height") == 0) {
			tm->tcfg->window_height = atoi(val);
		}
		if (strcmp(opt, "gtk_win_pos") == 0) {
			tm->tcfg->gtk_win_pos = atoi(val);
		}
		if (strcmp(opt, "window_title") == 0) {
			strcpy(tm->tcfg->window_title, val);
		}
		if (strcmp(opt, "maximized") == 0) {
			strcpy(tm->tcfg->maximized, val);
		}
		if (strcmp(opt, "remember_win_size") == 0) {
			strcpy(tm->tcfg->remember_win_size, val);
		}
		if (strcmp(opt, "fit_next") == 0) {
			strcpy(tm->tcfg->fit_next, val);
		}
		if (strcmp(opt, "remember_fit") == 0) {
			strcpy(tm->tcfg->remember_fit, val);
		}

		c = fgetc(fd);
	}

	fclose(fd);
}

void check_config (TM *tm) {
	struct passwd *pw = getpwuid(getuid());

	const char *homedir = pw->pw_dir;
	strcpy(tm->tcfg->homedir, homedir);
	strcat(tm->tcfg->homedir, "/"); /* will hold /home/user/ - including the ending slash */

	strcpy(tm->tcfg->config_file, tm->tcfg->homedir);
	strcat(tm->tcfg->config_file, ".neonview/neonview.cfg"); /* will hold /home/user/.neonview/neonview.cfg */

	struct stat st;
	char tmp1[4097];

	strcpy(tmp1, tm->tcfg->homedir);
	strcat(tmp1, ".neonview");

	if (stat(tmp1, &st) == 0) {
		// directory exists
		fprintf(stdout, "Found directory %s\n", tmp1);
		FILE *fd;
		fd = fopen(tm->tcfg->config_file, "rt");
		if (fd != NULL) {
			// success opening config for reading
			fprintf(stdout, "Found config file %s\n", tm->tcfg->config_file);
			fclose(fd);
			read_config(tm);
			if (strcmp(tm->tcfg->remember_fit, "yes") == 0) {
				tm->fit_images = 1;
			}
			// and now verify if the values read are valid
		} else {
			// config file could not be read
			// set default values for tcfg
			fprintf(stdout, "Could not read config file %s\n", tm->tcfg->config_file);
			set_default_values(tm);
			fd = fopen(tm->tcfg->config_file, "wt");
			if (fd != NULL) {
				// config opened for writing
				write_default(fd);
				fclose(fd);
				fprintf(stdout, "Successfully written default config file %s\n", tm->tcfg->config_file);
			} else {
				// could not open config for writing
				fprintf(stderr, "ERROR: Could not open config file %s for writing. Check disk space and file permissions. NeonView will run using default settings, but it will not be able to save any change.\n", tm->tcfg->config_file);
				set_default_values(tm);
			}
		}
	} else {
		// directory is missing or inaccessible
		fprintf(stdout, "Directory %s is missing or inaccessible\n", tmp1);
		if (mkdir(tmp1, 0777) == 0) {
			// success creating directory
			fprintf(stdout, "Successfully created directory %s\n", tmp1);
			FILE *fd;
			fd = fopen(tm->tcfg->config_file, "wt");
			if (fd != NULL) {
				// success creating file for writing
				// now we write the default config values
				write_default(fd);
				fprintf(stdout, "Successfully written default config file %s\n", tm->tcfg->config_file);
				set_default_values(tm);
				fclose(fd);
			} else {
				// config file couldn't be created
				// set default values for tcfg
				fprintf(stderr, "ERROR: Could not open config file %s for writing. Check disk space and file permissions. NeonView will continue using default settings, but it will not be able to save any changes.\n", tm->tcfg->config_file);
			set_default_values(tm);
			}
		} else {
			// directory couldn't be created
			// set default values for tcfg
			fprintf(stderr, "ERROR: Could not create directory %s. Check disk space and file permissions. NeonView will continue using default settings, but it will not be able to save any changes.\n", tmp1);
			set_default_values(tm);
		}
	}
}

void write_default (FILE *fd) {
	fprintf(fd, "# This file is updated automatically by NeonView.\n");
	fprintf(fd, "# Changes done via the Tools->Settings menu entry are reflected here.\n");
	fprintf(fd, "# You can edit this file manually too.\n\n");
	fprintf(fd, "window_width=800 # main window width, in pixels\n");
	fprintf(fd, "window_height=480 # main window height, in pixels\n");
	fprintf(fd, "gtk_win_pos=1 # main window position on screen (integer between 0-4, see the manual)\n");
	fprintf(fd, "maximized=no # start with main window maximized or not\n");
	fprintf(fd, "window_title=NeonView # main window title\n");
	fprintf(fd, "remember_win_size=yes # remember main window size between sessions\n");
	fprintf(fd, "fit_next=yes # fit all images to window after pressing the Fit to Window button\n");
	fprintf(fd, "remember_fit=no # remember the last choice - fit to window or normal size\n");
}

void set_default_values (TM *tm) {
	tm->tcfg->window_width = 800;
	tm->tcfg->window_height = 480;
	strcpy(tm->tcfg->maximized, "no");
	strcpy(tm->tcfg->remember_win_size, "yes");
	strcpy(tm->tcfg->window_title, "NeonView");
	tm->tcfg->gtk_win_pos = 1;
	strcpy(tm->tcfg->fit_next, "yes");
	strcpy(tm->tcfg->remember_fit, "no");
}

int write_config (TM *tm) {
	FILE *fd;
	fd = fopen(tm->tcfg->config_file, "wt");
	if (!fd) {
		fprintf(stderr, "ERROR: Could not open config file %s for writing. Check file permissions. NeonView is not able to save any configuration changes that you made.\n", tm->tcfg->config_file);
		return -1;
	}
	fprintf(fd, "# This file is updated automatically by NeonView.\n");
	fprintf(fd, "# Changes done via the Tools->Settings menu entry are reflected here.\n");
	fprintf(fd, "# You can edit this file manually too.\n\n");

	int width, height;
	char str1[257], str2[257];

	if (strcmp(tm->tcfg->remember_win_size, "yes") == 0) {
		gtk_window_get_size(GTK_WINDOW(tm->window), &width, &height);
		int2str(width, str1);
		int2str(height, str2);
	} else {
		int2str(tm->tcfg->window_width, str1);
		int2str(tm->tcfg->window_height, str2);
	}

	fprintf(fd, "window_width=%s # main window width, in pixels\n", str1);
	fprintf(fd, "window_height=%s # main window height, in pixels\n", str2);

	int2str(tm->tcfg->gtk_win_pos, str1);

	fprintf(fd, "gtk_win_pos=%s # main window position on screen (integer between 0-4, see the manual)\n", str1);

	if (strcmp(tm->tcfg->maximized, "yes") == 0) {
		fprintf(fd, "maximized=yes # start with main window maximized or not\n");
	} else {
		fprintf(fd, "maximized=no # start with main window maximized or not\n");
	}

	fprintf(fd, "window_title=%s # main window title\n", tm->tcfg->window_title);

	if (strcmp(tm->tcfg->remember_win_size, "yes") == 0) {
		fprintf(fd, "remember_win_size=yes # remember main window size between sessions\n");
	} else {
		fprintf(fd, "remember_win_size=no # remember main window size between sessions\n");
	}

	if (strcmp(tm->tcfg->fit_next, "yes") == 0) {
		fprintf(fd, "fit_next=yes # fit all images to window after pressing the Fit to Window button\n");
	} else {
		fprintf(fd, "fit_next=no # fit all images to window after pressing the Fit to Window button\n");
	}

	if (strcmp(tm->tcfg->remember_fit, "yes") == 0) {
		fprintf(fd, "remember_fit=yes # remember the last choice - fit to window or normal size\n");
	} else {
		fprintf(fd, "remember_fit=no # remember the last choice - fit to window or normal size\n");
	}

	return 0;
}

void gtk_my_quit (TM *tm) {
	write_config(tm);

	gtk_main_quit();
}
